//+------------------------------------------------------------------+
//|                                                   EasyObject.mqh |
//|                           https://www.mql5.com/en/users/icreator |
//|                                            ic.icreator@gmail.com |
//+------------------------------------------------------------------+
#property copyright "ic.icreator@gmail.com"
#property link "https://www.mql5.com/en/users/icreator"
#property strict
#property version "1.25"

//+------------------------------------------------------------------+
//| EasyObject class                                                 |
//+------------------------------------------------------------------+
class EasyObject
{
  private:
    string m_obj_name;
    long m_chart_id;
    ENUM_OBJECT m_obj_type;
    int m_sub_window;

    static EasyObject object; // singleton

    EasyObject(void)          // the contructor is private
    {
        m_chart_id = 0;
        m_sub_window = 0;
        m_obj_type = -1;
    };

  public:
    ~EasyObject(void){};

    // main static accessors
    
    static EasyObject *instance()
    {
        return &object;
    }
    
    static EasyObject *init(const string name, const ENUM_OBJECT type, const int window = 0, const long id = 0)
    {
        object.chartID(id);
        object.name(name);
        object.type(type);
        object.subWindow(window);
        return &object;
    }

    static EasyObject *bind(const string name, const ENUM_OBJECT type, const int window = 0, const long id = 0)
    {
        object.init(name, type, window, id);
        return object.create();
    }

    // setters
    
    EasyObject *reset()
    {
        m_obj_name = NULL;
        m_chart_id = 0;
        m_sub_window = 0;
        m_obj_type = -1;
        return &this;
    }

    EasyObject *chartID(const long id)
    {
        m_chart_id = id;
        return &this;
    }
    
    EasyObject *subWindow(const int win)
    {
        m_sub_window = win;
        return &this;
    }

    EasyObject *create()
    {
        ObjectCreate(chartID(), name(), type(), 0, 0, 0);
        return &this;
    }
    
    EasyObject *type(const ENUM_OBJECT type)
    {
        this.m_obj_type = type;
        return &this;
    }

    EasyObject *colour(const color inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_COLOR, inp);
        return &this;
    }

    EasyObject *style(const ENUM_LINE_STYLE inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_STYLE, inp);
        return &this;
    }

    EasyObject *width(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_WIDTH, inp);
        return &this;
    }

    EasyObject *back(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_BACK, inp);
        return &this;
    }

    EasyObject *zOrder(const long inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_ZORDER, inp);
        return &this;
    }

    EasyObject *fill(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_FILL, inp);
        return &this;
    }

    EasyObject *hidden(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_HIDDEN, inp);
        return &this;
    }

    EasyObject *selected(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_SELECTED, inp);
        return &this;
    }

    EasyObject *readOnly(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_READONLY, inp);
        return &this;
    }

    EasyObject *time(const datetime inp, const int anchor_point = 0)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_TIME, anchor_point, inp);
        return &this;
    }

    EasyObject *selectable(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_SELECTABLE, inp);
        return &this;
    }

    EasyObject *levels(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_LEVELS, inp);
        return &this;
    }

    EasyObject *levelColor(const color inp, const int level_number = 0)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_LEVELCOLOR, level_number, inp);
        return &this;
    }

    EasyObject *levelStyle(const ENUM_LINE_STYLE inp, const int level_number = 0)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_LEVELSTYLE, level_number, inp);
        return &this;
    }

    EasyObject *levelWidth(const int inp, const int level_number = 0)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_LEVELWIDTH, level_number, inp);
        return &this;
    }

    EasyObject *align(const ENUM_ALIGN_MODE inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_ALIGN, inp);
        return &this;
    }

    EasyObject *fontSize(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_FONTSIZE, inp);
        return &this;
    }

    EasyObject *ray(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_RAY, inp);
        return &this;
    }

    EasyObject *rayRight(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_RAY_RIGHT, inp);
        return &this;
    }

    EasyObject *ellipse(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_ELLIPSE, inp);
        return &this;
    }

    EasyObject *arrowCode(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_ARROWCODE, inp);
        return &this;
    }

    EasyObject *timeframes(const long inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_TIMEFRAMES, inp);
        return &this;
    }

    EasyObject *anchor(const ENUM_ARROW_ANCHOR inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_ANCHOR, inp);
        return &this;
    }

    EasyObject *anchor(const ENUM_ANCHOR_POINT inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_ANCHOR, inp);
        return &this;
    }

    EasyObject *x(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_XDISTANCE, inp);
        return &this;
    }

    EasyObject *y(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_YDISTANCE, inp);
        return &this;
    }

    EasyObject *direction(const ENUM_GANN_DIRECTION inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_DIRECTION, inp);
        return &this;
    }

    EasyObject *state(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_STATE, inp);
        return &this;
    }

    EasyObject *xSize(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_XSIZE, inp);
        return &this;
    }

    EasyObject *ySize(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_YSIZE, inp);
        return &this;
    }

    EasyObject *xOffset(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_XOFFSET, inp);
        return &this;
    }

    EasyObject *yOffset(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_YOFFSET, inp);
        return &this;
    }

    EasyObject *period(const ENUM_TIMEFRAMES inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_PERIOD, inp);
        return &this;
    }

    EasyObject *dateScale(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_DATE_SCALE, inp);
        return &this;
    }

    EasyObject *priceScale(const bool inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_PRICE_SCALE, inp);
        return &this;
    }

    EasyObject *chartScale(const int inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_CHART_SCALE, inp);
        return &this;
    }

    EasyObject *bgColor(const color inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_BGCOLOR, inp);
        return &this;
    }

    EasyObject *corner(const ENUM_BASE_CORNER inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_CORNER, inp);
        return &this;
    }

    EasyObject *borderType(const ENUM_BORDER_TYPE inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_BORDER_TYPE, inp);
        return &this;
    }

    EasyObject *borderColor(const color inp)
    {
        ObjectSetInteger(chartID(), name(), OBJPROP_BORDER_COLOR, inp);
        return &this;
    }

    EasyObject *price(const double price, const int anchor_point = 0)
    {
        ObjectSetDouble(0, name(), OBJPROP_PRICE, anchor_point, price);
        return &this;
    }

    EasyObject *levelValue(const double inp, const int level_number = 0)
    {
        ObjectSetDouble(chartID(), name(), OBJPROP_LEVELVALUE, level_number, inp);
        return &this;
    }

    EasyObject *scale(const double inp)
    {
        ObjectSetDouble(chartID(), name(), OBJPROP_SCALE, inp);
        return &this;
    }

    EasyObject *angle(const double inp)
    {
        ObjectSetDouble(chartID(), name(), OBJPROP_ANGLE, inp);
        return &this;
    }

    EasyObject *deviation(const double inp)
    {
        ObjectSetDouble(chartID(), name(), OBJPROP_DEVIATION, inp);
        return &this;
    }

    EasyObject *name(const string name)
    {
        this.m_obj_name = name;
        return &this;
    }

    EasyObject *text(const string inp)
    {
        ObjectSetString(chartID(), name(), OBJPROP_TEXT, inp);
        return &this;
    }

    EasyObject *tooltip(const string inp)
    {
        ObjectSetString(chartID(), name(), OBJPROP_TOOLTIP, inp);
        return &this;
    }

    EasyObject *levelText(const string inp, const int level_number = 0)
    {
        ObjectSetString(chartID(), name(), OBJPROP_LEVELTEXT, level_number, inp);
        return &this;
    }

    EasyObject *font(const string inp)
    {
        ObjectSetString(chartID(), name(), OBJPROP_FONT, inp);
        return &this;
    }

    EasyObject *bmpFile(const string inp, const int state = 0)
    {
        ObjectSetString(chartID(), name(), OBJPROP_BMPFILE, state, inp);
        return &this;
    }

    EasyObject *symbol(const string inp)
    {
        ObjectSetString(chartID(), name(), OBJPROP_SYMBOL, inp);
        return &this;
    }

    EasyObject *move(const datetime time, const double price, const int point_index = 0)
    {
        ObjectMove(chartID(), name(), point_index, time, price);
        return &this;
    }

    // getters

    string name() const
    {
        return m_obj_name;
    }

    ENUM_OBJECT type() const
    {
        return m_obj_type;
    }

    long chartID() const
    {
        return m_chart_id;
    }

    color colour() const
    {
        return (color)ObjectGetInteger(chartID(), name(), OBJPROP_COLOR);
    }

    ENUM_LINE_STYLE style() const
    {
        return (ENUM_LINE_STYLE)ObjectGetInteger(chartID(), name(), OBJPROP_STYLE);
    }

    int width() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_WIDTH);
    }

    bool back() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_BACK);
    }

    long zOrder() const
    {
        return ObjectGetInteger(chartID(), name(), OBJPROP_ZORDER);
    }

    bool fill() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_FILL);
    }

    bool hidden() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_HIDDEN);
    }

    bool selected() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_SELECTED);
    }

    bool readOnly() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_READONLY);
    }

    datetime time() const
    {
        return (datetime)ObjectGetInteger(chartID(), name(), OBJPROP_TIME);
    }

    bool selectable() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_SELECTABLE);
    }

    int levels() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_LEVELS);
    }

    color levelColor() const
    {
        return (color)ObjectGetInteger(chartID(), name(), OBJPROP_LEVELCOLOR);
    }

    ENUM_LINE_STYLE levelStyle() const
    {
        return (ENUM_LINE_STYLE)ObjectGetInteger(chartID(), name(), OBJPROP_LEVELSTYLE);
    }

    int levelWidth() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_LEVELWIDTH);
    }

    ENUM_ALIGN_MODE align() const
    {
        return (ENUM_ALIGN_MODE)ObjectGetInteger(chartID(), name(), OBJPROP_ALIGN);
    }

    int fontSize() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_FONTSIZE);
    }

    bool ray() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_RAY);
    }

    bool rayRight() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_RAY_RIGHT);
    }

    bool ellipse() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_ELLIPSE);
    }

    char arrowCode() const
    {
        return (char)ObjectGetInteger(chartID(), name(), OBJPROP_ARROWCODE);
    }

    long timeframes() const
    {
        return ObjectGetInteger(chartID(), name(), OBJPROP_TIMEFRAMES);
    }

    ENUM_ANCHOR_POINT anchor() const
    {
        return (ENUM_ANCHOR_POINT)ObjectGetInteger(chartID(), name(), OBJPROP_ANCHOR);
    }

    int x() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_XDISTANCE);
    }

    int y() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_YDISTANCE);
    }

    ENUM_GANN_DIRECTION direction() const
    {
        return (ENUM_GANN_DIRECTION)ObjectGetInteger(chartID(), name(), OBJPROP_DIRECTION);
    }

    bool state() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_STATE);
    }

    int xSize() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_XSIZE);
    }

    int ySize() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_YSIZE);
    }

    int xOffset() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_XOFFSET);
    }

    int yOffset() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_YOFFSET);
    }

    ENUM_TIMEFRAMES period() const
    {
        return (ENUM_TIMEFRAMES)ObjectGetInteger(chartID(), name(), OBJPROP_PERIOD);
    }

    bool dateScale() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_DATE_SCALE);
    }

    bool priceScale() const
    {
        return (bool)ObjectGetInteger(chartID(), name(), OBJPROP_PRICE_SCALE);
    }

    int chartScale() const
    {
        return (int)ObjectGetInteger(chartID(), name(), OBJPROP_CHART_SCALE);
    }

    color bgColor() const
    {
        return (color)ObjectGetInteger(chartID(), name(), OBJPROP_BGCOLOR);
    }

    ENUM_BASE_CORNER corner() const
    {
        return (ENUM_BASE_CORNER)ObjectGetInteger(chartID(), name(), OBJPROP_CORNER);
    }

    ENUM_BORDER_TYPE borderType() const
    {
        return (ENUM_BORDER_TYPE)ObjectGetInteger(chartID(), name(), OBJPROP_BORDER_TYPE);
    }

    double price() const
    {
        return ObjectGetDouble(chartID(), name(), OBJPROP_PRICE);
    }

    double levelValue() const
    {
        return ObjectGetDouble(chartID(), name(), OBJPROP_LEVELVALUE);
    }

    double scale() const
    {
        return ObjectGetDouble(chartID(), name(), OBJPROP_SCALE);
    }

    double angle() const
    {
        return ObjectGetDouble(chartID(), name(), OBJPROP_ANGLE);
    }

    double deviation() const
    {
        return ObjectGetDouble(chartID(), name(), OBJPROP_DEVIATION);
    }

    string text() const
    {
        return ObjectGetString(chartID(), name(), OBJPROP_TEXT);
    }

    string tooltip() const
    {
        return ObjectGetString(chartID(), name(), OBJPROP_TOOLTIP);
    }

    string levelText() const
    {
        return ObjectGetString(chartID(), name(), OBJPROP_LEVELTEXT);
    }

    string font() const
    {
        return ObjectGetString(chartID(), name(), OBJPROP_FONT);
    }

    string bmpFile() const
    {
        return ObjectGetString(chartID(), name(), OBJPROP_BMPFILE);
    }

    string symbol() const
    {
        return ObjectGetString(chartID(), name(), OBJPROP_SYMBOL);
    }

    EasyObject *priceToY(const double price)
    {
        int x, y;
        ChartTimePriceToXY(chartID(), 0, TimeCurrent(), price, x, y);
        ENUM_ANCHOR_POINT point = anchor();
        return this.y(y + (point == ANCHOR_RIGHT_LOWER ? -4 : 0));
    }

    EasyObject *XYToPrice(const int x, const int y)
    {
        datetime time;
        double price;
        ChartXYToTimePrice(chartID(), x, y, m_sub_window, time, price);
        this.price(price);
        this.time(time);

        return &this;
    }

    bool exists() const
    {
        if(ObjectFind(chartID(), name()) >= 0)
            return true;
        return false;
    }

    int priceToY() const
    {
        int x, y;
        ChartTimePriceToXY(chartID(), 0, TimeCurrent(), price(), x, y);
        return y;
    }

    void XY(int &x, int &y) const
    {
        datetime time = time();
        if(time == 0) time = iTime(_Symbol, _Period, 0);
        ChartTimePriceToXY(chartID(), 0, time, price(), x, y);
    }

    bool remove() const
    {
        return ObjectDelete(chartID(), name());
    }
    
};

// defines the single instance of the class
static EasyObject EasyObject::object;

//+------------------------------------------------------------------+

/*
    // Example of usage:
    
    EasyObject::bind("name", OBJ_TEXT)
    .text("Hello, World!")
    .font("Calibri")
    .fontSize(16)
    .colour(clrLimeGreen)
    .anchor(ANCHOR_LEFT_LOWER)
    .time(TimeCurrent())
    .price(SymbolInfoDouble(_Symbol, SYMBOL_ASK));

*/